#ifndef _MATRIX_H_
#define _MATRIX_H_

#include <unistd.h>
#include <stdio.h>
#include <math.h>
#include <iostream.h>
#include <iomanip.h>

typedef unsigned int uint;

class Vector;
class Matrix;
class MatrixException;

class MatrixException{
	private:
	char *msg_;

	public:
	MatrixException(char *msg = "unspecified matrix exception");
	friend ostream& operator<<(ostream& s, const MatrixException& e);
};

class Vector{
	private:
	double data_[4];
	double& x_;
	double& y_;
	double& z_;
	double& h_;

	public:
	const uint X, Y, Z, H, ROWS;
	Vector(double x = 0, double y = 0, double z = 0, double h = 1);
	//Vector(const Vector& v);
	~Vector();
	// member access
	double  x() const { return x_; }
	double& x()				{ return x_; }
	double  y() const { return y_; }
	double& y()				{ return y_; }
	double  z() const { return z_; }
	double& z()				{ return z_; }
	double  h() const { return h_; }
	double& h()				{ return h_; }
	// overloaded operators
	double& operator()(uint row);					// v(i) = x
	double  operator()(uint row) const;		// x = v(i)
	Vector& operator=(const Vector& v);		// assignment
	Vector  operator+() const;						// unary ops
	Vector  operator-() const;
	Vector  operator+(const Vector& v);		// binary ops
	Vector  operator-(const Vector& v);
	Vector  operator*(const Vector& v);
	Vector  operator*(double d);
	Vector  operator/(const Vector& v);
	Vector  operator/(double d);
	Vector  operator==(const Vector& v);
	// operations
	double magnitude();
	Vector normalize();
	double dotProduct(const Vector& v);
	Vector crossProduct(const Vector& v);
	friend ostream& operator<<(ostream& s, const Vector& v);
};

class Matrix{
	private:
	double data_[4 * 4];

	public:
	const uint ROWS, COLS;
	Matrix();
	//Matrix(const Matrix& m);							// Copy constructor
	~Matrix();

	// misc
	bool checkBounds();

	// fundamental operations
	void identity();
	void invert();
	void transpose();
	double determinant() const;
	// create various types of matrices
	static Matrix identityMatrix();
	static Matrix invert(const Matrix& m);
	static Matrix transpose(const Matrix& m);
	static double determinant(const Matrix& m);
	static Matrix translateMatrix(const Vector& v);
	static Matrix scaleMatrix(double d);
	static Matrix rotateXMatrix(double angle);			// pitch
	static Matrix rotateYMatrix(double angle);			// yaw
	static Matrix rotateZMatrix(double angle);			// roll
	static Matrix rotateXYZMatrix(const Vector& v);	// pitch, yaw, roll
	// shear()
	//Matrix& transpose(Matrix& m); // handle assignment to `this'!
	double& operator()(uint row, uint col);
	double  operator()(uint row, uint col) const;
	Matrix& operator=(const Matrix& m);
	Matrix  operator*(const Matrix& m) const;
	Vector  operator*(const Vector& v) const;
	friend ostream& operator<<(ostream& s, const Matrix& m);
};

/* generic vector(N) and matrix(M, N)
class Vector{
	private:
	uint rows_;
	double *data_;

	public:
	Vector(uint rows = 4);
	Vector(double x, double y, double z);	// short-cut init for 3d-vector
	~Vector();
	// member access
	uint rows() const { return rows_; }
	// overloaded operators
	double& operator()(uint row);					// v(i) = x
	double  operator()(uint row) const;		// x = v(i)
	Vector& operator=(Vector& m);					// assignment
	Vector  operator+() const;						// unary ops
	Vector  operator-() const;
	Vector  operator+(const Vector& v);		// binary ops
	Vector  operator-(const Vector& v);
	Vector  operator*(const Vector& v);
	Vector  operator*(double d);
	Vector  operator/(const Vector& v);
	Vector  operator/(double d);
	Vector  operator==(const Vector& v);
	// operations
	double magnitude();
	Vector normalize();
	double dotProduct(const Vector& v);
	Vector crossProduct(const Vector& v);
};

class Matrix{
	private:
	uint rows_, cols_;
	double* data_;

	public:
	Matrix(uint rows = 4, uint cols = 4);
	//Matrix(const Matrix& m);							// Copy constructor
	~Matrix();

	// misc
	void print();
	bool checkBounds();
	bool matchSize(const Matrix &m);

	// operations
	void identity();
	void inverse() { }
	Matrix& transpose();
	// translate()
	// scale()
	// rotate(x, y, z) // pitch, yaw, roll
	// shear()
	//Matrix& transpose(Matrix& m); // handle assignment to `this'!
	double& operator()(uint row, uint col);
	double  operator()(uint row, uint col) const;
	Matrix& operator=(const Matrix& m);
	Matrix& operator*(const Matrix& m);
	Vector& operator*(Vector& v);
};
*/

#endif _MATRIX_H_
